// Here goes prototypes declaration and documentation

#include <windows.h>
#include "define.h" /* this include file stores typedef that can be easily used */


#ifdef __cplusplus 
	extern "C" {
#endif


/** \defgroup PCIe3521 xPCIe-3521 functions
 * @{
 *		\addtogroup General xPCIe-3521 general functions
 *		\ingroup PCIe3521
 *		@{ 
 */
 
/**  Enables to get the number of xPCIe-3521 boards present in the system. \n
 * The function fill the value pointed by pb_NbrOfBoards with the number of boards present.
 *                                                                            
 * @param[out] pb_NbrOfBoards Value that will be filled with the number of boards     
 * @return	0 : No error \n
 *		   -1 : Error while getting the number of boards \n
 */
_INT_ i_PCIe3521_GetNumberOfBoards (PBYTE	pb_NbrOfBoards);

/** Get device information of the board.
 *                                                                   
 * @param[in] b_BoardIndex : Index of the board
 * @param[in] dw_IdentifierStringSize : size of the identifier parameter 
 * @param[out] pc_Identifier: Identifier of the board
 * @param[out] pdw_UINumber : This number is typically a user-perceived slot number, such as a number printed next to the slot on the board, or some other number that makes locating the physical device easier for the user. If the device is on a bus that has no UI number convention, or if the bus driver for the device cannot determine the UI number, this value is 0xFFFFFFFF.
 * @param[out] pdw_DeviceNumber : Device number.
 * @param[out] pdw_BusNumber : Bus number.
 * @param[out] pdw_BaseAddress0 : Base address 0 of the board
 * @param[out] pdw_BaseAddress1 : Base address 1 of the board
 * @param[out] pdw_BaseAddress2 : Base address 2 of the board
 * @param[out] pdw_BaseAddress3 : Base address 3 of the board
 * @param[out] pb_Interrupt : Interrupt line of the board
 * @return	 0 : No error \n
 *			-1 : Unable to open the board via index \n
 *			-2 : Error when calling driver's IOCTL \n
 */
_INT_ i_PCIe3521_GetBoardInformation(BYTE		b_BoardIndex,
									DWORD		dw_IdentifierStringSize,
									PCHAR		pc_Identifier,
									PDWORD		pdw_UINumber,
									PDWORD		pdw_DeviceNumber,
									PDWORD		pdw_BusNumber,
									PDWORD		pdw_BaseAddress0,
									PDWORD		pdw_BaseAddress1,
									PDWORD		pdw_BaseAddress2,
									PDWORD		pdw_BaseAddress3,
									PBYTE		pb_Interrupt);

/** Get revision of the board.
 *
 * @param[in] b_BoardIndex : Index of the board
 * @param[out] pdw_BoardRevision: Revision number of the board
 * @return	 0 : No error \n
 *			-1 : Unable to open the board via index \n
 *			-2 : Error when calling driver's IOCTL \n
 */
_INT_ i_PCIe3521_GetBoardRevision(BYTE		b_BoardIndex,
									PDWORD		pdw_BoardRevision);

/** Open the board with the index : b_BoardIndex. A handle is returned to the \n    
 * user which allows to use the driver functions. \n 
 *                                                                                  
 * @param[in] b_BoardIndex : Index of the board to open                             
 * @param[out] ph_DeviceHandle : Handle of the selected xPCIe-3521 board for using the functions
 * @return		0 : No error
 *             -1 : Unable to get the number of available boards \n                                    
 *             -2 : Wrong device index parameter. The parameter is too high \n
 *             -3 : Error while getting the device's class \n
 *             -4 : Error while getting informations about the device \n
 *             -5 : Memory allocation error \n
 *             -6 : Get device interface detail error \n
 *             -7 : Error while creating communication handle \n
 *			   -8 : Error in the initialization callback function \n
 */
_INT_ i_PCIe3521_OpenBoardViaIndex (BYTE	b_BoardIndex,
								   PHANDLE	ph_DeviceHandle);    

/** Open the board with the indentifier : pc_Identifier. A handle is returned to the \n    
 * user which allows to use the following functions. \n                                                         
 *                                                                                  
 * @param[in] pc_Identifier : Identifier of the board to open
 * @param[out] ph_DeviceHandle : Handle of the board xPCIe-3521 for using the functions
 * @return	 0 : No Error \n
			-1 : Unable to get the number of available boards \n  
			-2 : Unable to get the board identifier \n
			-3 : Unable to open the board \n
			-4 : Identifier not found \n
 */
_INT_ i_PCIe3521_OpenBoardViaIdentifier (PCHAR	pc_Identifier,
										PHANDLE	ph_DeviceHandle);
										
/** Release the handle of the board. Block the access to the board.
 *                                                                   
 * @param[in] h_DeviceHandle    : Handle of the board                
 * @return	 0 : No Error \n
 *			-1 : The device handle parameter is wrong \n
 */
_INT_ i_PCIe3521_CloseBoard (HANDLE	h_DeviceHandle);

/** Return the version of the DLL.
 *                                                                   
 * @param[out] pdw_Version  : Version. sample : 0x01010910 -> version 1.01 september 2010       
 * @return	0 : No Error \n
 */
_INT_ i_PCIe3521_GetDLLVersion (PDWORD	pdw_Version);

/*Function's Name   : i_PCIe3521_GetDeviceDriverVersion*/
/** Return the version of the device driver.
*                                                                   
* @param[in] h_DeviceHandle  : Handle of the board                
* @param[out] pdw_Version  : Version. sample : 0x01011211 -> version 1.01 - 12/2011
* @return    0 : No Error \n                                                     
*             -1 : Error while calling IOCTL function \n
*/
_INT_ i_PCIe3521_GetDeviceDriverVersion(HANDLE h_DeviceHandle, PDWORD pdw_Version);

/**
 *		@}
 *		
 *		\addtogroup Interrupt xPCIe-3521 interrupt management functions
 * 		\ingroup PCIe3521
 *		@{ 
 */
 
/** Install an interrupt callback function.
 * This function will be called each time the board generate an interruption. \n
 * The value of dw_InterruptSource will enable you to determine the source of the interruption:
 * <ul>
 *	<li>0x1: End of sequence interrupt. When you use i_PCIe3521_Read1AnalogInput and i_PCIe3521_ReadMoreAnalogInputs with interrupt, you will receive this interrupt. The values are then available thanks to pw_Args parameter</li>
 *	<li>0x2: Timer interrupt. </li>
 *	<li>0x4: DMA interrupt. You will receive this interrupt when using i_PCIe3521_InitAndStartAnalogInputSequence. When you get this interrupt, you have to read the data measured by the board by calling the function l_PCIe3521_ReadSequenceData (see its own documentation). If your computer is too slow to handle all the data, after a while you will have an internal data fifo overflow (dw_InterruptMask will be 0x4). You can change the size of the internal data fifo in the driver's configuration pages.</li>
 *	<li>0x8: End of DMA acquisition. You will receive this interrupt when using i_PCIe3521_InitAndStartAnalogInputSequence in non continuous mode</li>
 *	<li>0x10: PLD FIFO overflow. The board is too slow to handle all the data</li>
 * </ul> 
 *
 * @param[in] h_DeviceHandle    : Handle of the board                
 * @param[in] v_FunctionName    : Address of the callback function
 *		\n	\b h_DeviceHandle	: Handle of the board
 *		\n	\b b_FIFOOverflow	: If uc_FIFOOverflow != 0, there are too many untreated interrupts in the kernel FIFO, and any new information is lost
 *		\n	\b dw_InterruptSource : Source of the interrupt
 *		\n	\b dw_InterruptMask	: Source of the interrupt
 *		\n	\b pw_Args			: Values read during the interrupt. If DMA interrupt, this pointer will be set to NULL.
 *		\n	\b dw_ArgsCount		: Number of values read during the interrupt
 *
 * @return	 0 : No Error
 *			-1 : The device handle parameter is wrong \n
 *			-2 : An interrupt routine was still isntalled. Remove it with i_PCIe3521_ResetBoardIntRoutine and retry \n
 *			-3 : Error while creating kill event \n
 *			-4 : Error while creating interrupt event \n
 *			-5 : Error while creating interrupt thread \n
 *			-6 : Error while changing the priority class of the thread \n
 *			-7 : Error while changing the priority of the thread \n
 *			-8 : Error while calling the IOCTL command \n
 *			-9 : Error in the interrupt callback function \n
 */
_INT_ i_PCIe3521_SetBoardIntRoutine (HANDLE	h_DeviceHandle,
									void	(*v_FunctionName)(	HANDLE	h_DeviceHandle,
																BYTE	b_FIFOOverflow,
																DWORD	dw_InterruptSource,
																DWORD	dw_InterruptMask,
																PWORD	pw_Args,
																DWORD	dw_ArgsCount)
);

/** Deinstall the interrupt callback function.
 *
 * @param[in] h_DeviceHandle    : Handle of the board                
 * @return	 0 : No Error
 *			-1 : The device handle parameter is wrong \n
 *			-2 : No interrupt routine installed \n
 *			-3 : Error in the reset interrupt callback function \n
 */
_INT_ i_PCIe3521_ResetBoardIntRoutine (HANDLE	h_DeviceHandle);


/**
 *		@}
 *		
 *		\addtogroup Digital xPCIe-3521 digital inputs/outputs functions
 * 		\ingroup PCIe3521
 *		@{ 
 */

/** Read 1 digital input of the board
 *                                                                          
 * @param[in] h_DeviceHandle Handle of the board                      
 * @param[in] b_Channel Channel to read (0 to 3)
 * @param[out] pb_Value Status of the digital input
 *
 * @return	 0 : No Error \n                                             
 *			-1 : Invalid channel \n           
 *			-2 : Error while calling IOCTL function (wrong handle ?) \n
 */
_INT_ i_PCIe3521_Read1DigitalInput(HANDLE	h_DeviceHandle,
								  BYTE		b_Channel,
								  PBYTE		pb_Value);

/** Read the 4 digital inputs of the board
 *                                                                          
 * @param[in] h_DeviceHandle Handle of the board
 * @param[out] pb_Value Status of the digital inputs
 *
 * @return	 0 : No Error \n                                             
 *			-1 : Error while calling IOCTL function (wrong handle ?) \n
 */
_INT_ i_PCIe3521_Read4DigitalInputs(HANDLE	h_DeviceHandle,
								   PBYTE	pb_Value);

/** Set 1 digital output of the board to HIGH
 *                                                                          
 * @param[in] h_DeviceHandle Handle of the board                      
 * @param[in] b_Channel Channel to set (0 to 3)
 *
 * @return	 0 : No Error \n                                             
 *			-1 : Wrong channel \n           
 *			-3 : Error while calling IOCTL function (wrong handle ?) \n
 */
_INT_ i_PCIe3521_Set1DigitalOutputOn(HANDLE	h_DeviceHandle,
								  BYTE		b_Channel);

/** Set 1 digital output of the board to LOW
 *                                                                          
 * @param[in] h_DeviceHandle Handle of the board                      
 * @param[in] b_Channel Channel to reset (0 to 3)
 *
 * @return	 0 : No Error \n                                             
 *			-1 : Wrong channel \n          
 *			-3 : Error while calling IOCTL function (wrong handle ?) \n
 */
_INT_ i_PCIe3521_Set1DigitalOutputOff(HANDLE	h_DeviceHandle,
									 BYTE	b_Channel);

/** Set the digital outputs set in the mask to HIGH
 *                                                                          
 * @param[in] h_DeviceHandle Handle of the board                      
 * @param[in] b_Mask Channels to set (0x0 to 0xF)
 *
 * @return	 0 : No Error \n                                             
 *			-1 : Wrong mask \n         
 *			-3 : Error while calling IOCTL function (wrong handle ?) \n
 */
_INT_ i_PCIe3521_Set4DigitalOutputsOn(HANDLE	h_DeviceHandle,
									 BYTE	b_Mask);

/** Set the digital outputs set in the mask to LOW
 *                                                                          
 * @param[in] h_DeviceHandle Handle of the board                      
 * @param[in] b_Mask Channels to reset (0x0 to 0xF)
 *
 * @return	 0 : No Error \n                                             
 *			-1 : Wrong mask \n             
 *			-3 : Error while calling IOCTL function (wrong handle ?) \n
 */
_INT_ i_PCIe3521_Set4DigitalOutputsOff(HANDLE	h_DeviceHandle,
									  BYTE	b_Mask);

/** Enable the digital output memory \n
 * When digital output memory is enabled, the active channels stay active until calling i_PCIe3521_Set1DigitalOutputOff or i_PCIe3521_Set4DigitalOutputsOff. \n
 * When digital output memory is disabled, the active channels are reset at each call of i_PCIe3521_Set1DigitalOutputOn or i_PCIe3521_Set4DigitalOutputsOn.
 *                                                                          
 * @param[in] h_DeviceHandle Handle of the board
 *
 * @return	 0 : No Error \n                                             
 *			-1 : Error while calling IOCTL function (wrong handle ?) \n
 */
_INT_ i_PCIe3521_SetDigitalOutputMemoryOn(HANDLE h_DeviceHandle);

/** Disable the digital output memory \n
 * When digital output memory is enabled, the active channels stay active until calling i_PCIe3521_Set1DigitalOutputOff or i_PCIe3521_Set4DigitalOutputsOff. \n
 * When digital output memory is disabled, the active channels are reset at each call of i_PCIe3521_Set1DigitalOutputOn or i_PCIe3521_Set4DigitalOutputsOn.
 *                                                                          
 * @param[in] h_DeviceHandle Handle of the board
 *
 * @return	 0 : No Error \n                                             
 *			-1 : 
 */
_INT_ i_PCIe3521_SetDigitalOutputMemoryOff(HANDLE h_DeviceHandle);

/** Get the status of 1 digital output of the board
 *                                                                          
 * @param[in] h_DeviceHandle Handle of the board
 * @param[in] b_Channel Channel to get (0 to 3)
 * @param[out] pb_OutputsStatus Status of the digital output
 *
 * @return	 0 : No Error \n                                             
 *			-1 : Wrong channel \n
 *			-2 : Error while calling IOCTL function (wrong handle ?) \n
 */
_INT_ i_PCIe3521_Get1DigitalOutputStatus(HANDLE	h_DeviceHandle,
										BYTE	b_Channel,
										PBYTE	pb_OutputsStatus);

/** Get the status of all the digital outputs of the board
 *                                                                          
 * @param[in] h_DeviceHandle Handle of the board
 * @param[out] pb_OutputsStatus Status of the digital output
 *
 * @return	 0 : No Error \n                                             
 *			-1 : Error while calling IOCTL function (wrong handle ?) \n
 */
_INT_ i_PCIe3521_Get4DigitalOutputsStatus(HANDLE	h_DeviceHandle,
										 PBYTE	pb_OutputsStatus);

/**
 *		@}
 *		
 *		\addtogroup Timer xPCIe-3521 Timer functions
 * 		\ingroup PCIe3521
 *		@{ 
 */

/**
* Enables to start the timer. \n
* @param[in] h_DeviceHandle Handle of the board
* @param[in] b_TimerNb Specify the timer to be used. (0 or 1). \n
* @param[in] b_TimerMode Specify the mode of the timer. (PCIe3521_TIMER_MODE_2, PCIe3521_TIMER_MODE_3). \n
* @param[in] b_TimeBase Select the time base (PCIe3521_TIME_BASE_MICROSECONDS, PCIe3521_TIME_BASE_MILLISECONDS, PCIe3521_TIME_BASE_SECONDS)
* @param[in] b_OutputMode Set the way the output will change (PCIe3521_OUTPUT_ENABLED, PCIe3521_OUTPUT_DISABLED, PCIe3521_OUTPUT_INVERTED)
* @param[in] b_InterruptEnabled Enable to use or not use an interrupt (0,1).
* @param[in] w_ReloadValue The time to count (depend on b_TimeBase)
* @param[in] b_WatchdogEnabled Specify if the watchdog is used (0,1). \n
* PCIe3521_TIMER_MODE_2: It functions like a divide-by-w_ReloadValue counter. It is used to generate a real time clock interrupt. OUT is initially high after the initialization. When the initial count has decremented to 1, OUT goes low for one CLK pulse. OUT then goes high again, the counter reloads the initial count (w_ReloadValue) and the process is repeated. This action can generate an interrupt. \n
* PCIe3521_TIMER_MODE_3: This mode is similar to mode 2 except for the duty cycle of OUT. OUT will initially be high after the initialisation. When the initial count (ul_ReloadValue) has expired, OUT goes low for the next sequence. When the initial count (ul_ReloadValue) of that sequence has expired, OUT goes high again for the following sequence. The mode is periodic; the sequence above is repeated indefinitely. \n
* @return	 0 : success \n
*           -1 : b_TimerNb is invalid \n
*		    -2 : b_TimerMode is invalid \n
*			-3 : b_TimeBase is invalid \n
*		    -4 : b_OutputMode is invalid \n
*			-5 : b_InterruptEnabled is invalid \n
*           -6 : b_WatchdogEnabled is invalid \n
*		    -7 : Error while calling IOCTL function (wrong handle ?)
**/
_INT_ i_PCIe3521_InitAndStartTimer(HANDLE h_DeviceHandle,
                                   BYTE b_TimerNb,
                                   BYTE b_TimerMode,
                                   BYTE b_TimeBase,
                                   BYTE b_OutputMode,
                                   BYTE b_InterruptEnabled,
                                   WORD w_ReloadValue,
                                   BYTE b_WatchdogEnabled);

/** Stops the timer \n
 *
 * @param[in] h_DeviceHandle Handle of the board
 * @param[in] b_TimerNb Specify the timer to be used. (0 or 1). \n
 *
 * @return	 0 : No Error \n
 *          -1 : b_TimerNb is invalid \n
 *			-2 : Error while calling IOCTL function (wrong handle ?) \n
 */
_INT_ i_PCIe3521_StopTimer(HANDLE h_DeviceHandle, BYTE b_TimerNb);

/** Reads the timer status \n
 *
 * @param[in] h_DeviceHandle Handle of the board
 * @param[in] b_TimerNb Specify the timer to be used. (0 or 1). \n
 * @param[out] pb_TimerStatus Timer status (0 = disabled, 1 = enabled)
 * @param[out] pb_SoftwareTriggerStatus Status of the software trigger (0 = no software trigger occured, 1 = software trigger occured)
 * @param[out] pw_TimerValue Current value of the timer
 *
 * @return	 0 : No Error \n
 *          -1 : b_TimerNb is invalid \n
 *			-2 : Error while calling IOCTL function (wrong handle ?) \n
 */
_INT_ i_PCIe3521_ReadTimerStatus(HANDLE	h_DeviceHandle,
								 BYTE   b_TimerNb,
                                 PBYTE	pb_TimerStatus,
                                 PBYTE	pb_SoftwareTriggerStatus,
                                 PWORD	pw_TimerValue);

/** Reads the timer status Extended \n
 *
 * @param[in] h_DeviceHandle Handle of the board
 * @param[in] b_TimerNb Specify the timer to be used. (0 or 1). \n
 * @param[out] pb_TimerStatus Timer status (0 = disabled, 1 = enabled)
 * @param[out] pb_Overflow (1 if the timer reached 0, else 0 )
 * @param[out] pb_SoftwareTriggerStatus Status of the software trigger (0 = no software trigger occured, 1 = software trigger occured)
 * @param[out] pw_TimerValue Current value of the timer
 *
 * @return	 0 : No Error \n
 *          -1 : b_TimerNb is invalid \n
 *			-2 : Error while calling IOCTL function (wrong handle ?) \n
 */
_INT_ i_PCIe3521_ReadTimerStatusEx(HANDLE	h_DeviceHandle,
								 BYTE   b_TimerNb,
                                 PBYTE	pb_TimerStatus,
								 PBYTE	pb_Overflow,
                                 PBYTE	pb_SoftwareTriggerStatus,
                                 PWORD	pw_TimerValue);


/** Read the value of the timer \n
 * 
 * @param[in] h_DeviceHandle Handle of the board
 * @param[in] b_TimerNb Specify the timer to be used. (0 or 1). \n
 * @param[out] pw_TimerValue Current value of the timer
 *
 * @return	 0 : No error \n
 *          -1 : b_TimerNb is invalid \n
 *		    -2 : Error while calling IOCTL function (wrong handle ?) \n
 */
_INT_ i_PCIe3521_ReadTimerValue(HANDLE	h_DeviceHandle,
                                BYTE    b_TimerNb,
								PWORD	pw_TimerValue);

/** Triggers the timer \n
 *
 * @param[in] h_DeviceHandle Handle of the board
 * @param[in] b_TimerNb Specify the timer to be used. (0 or 1). \n
 *
 * @return	 0 : No Error \n
 *          -1 : b_TimerNb is invalid \n
 *			-2 : Error while calling IOCTL function (wrong handle ?) \n
 */
_INT_ i_PCIe3521_TriggerTimer(HANDLE h_DeviceHandle, BYTE b_TimerNb);

/**
 *		@}
 *		
 *		\addtogroup Analog xPCIe-3521 Analog outputs functions
 * 		\ingroup PCIe3521
 *		@{ 
 */
 
/** Set the value of one analog output
 *                                                                          
 * @param[in] h_DeviceHandle Handle of the board                      
 * @param[in] uc_Channel Define the channel to set
 * @param[in] uc_Polarity Define the polarity for the channel (PCIe3521_OUTPUT_MODE_UNIPOLAR or PCIe3521_OUTPUT_MODE_BIPOLAR)
 * @param[in] us_Value Value to be set on the analog output
 *
 * @return	 0 : No Error \n                                             
 *			-1 : The device handle parameter is wrong \n
 *			-3 : Wrong channel \n
 *			-4 : Wrong polarity \n
 *			-5 : Error while calling IOCTL function (wrong handle ?) \n
 */
_INT_ i_PCIe3521_Write1AnalogOutput (
	HANDLE	h_DeviceHandle,
	UCHAR	uc_Channel, 
	UCHAR	uc_Polarity,
	USHORT	us_Value);
 
 /** Set the value of several analog outputs
 *                                                                          
 * @param[in] h_DeviceHandle Handle of the board                      
 * @param[in] uc_ChannelNbr Define the number of channels to set
 * @param[in] puc_Channel Pointer to an array containing the list of channels to be set
 * @param[in] puc_Polarity Pointer to an array containing the list of polarities for the defined channels (PCIe3521_OUTPUT_MODE_UNIPOLAR or PCIe3521_OUTPUT_MODE_BIPOLAR)
 * @param[in] pus_Value Pointer to an array containing the values to be set
 *
 * @return	 0 : No Error \n                                             
 *			-1 : The device handle parameter is wrong \n
 *			-2 : Wrong channel count \n
 *			-3 : Wrong channel \n
 *			-4 : Wrong polarity \n
 *			-5 : Error while calling IOCTL function (wrong handle ?) \n
 */
 _INT_ i_PCIe3521_WriteMoreAnalogOutputs(
	HANDLE	h_DeviceHandle, 
	UCHAR	uc_ChannelNbr, 
	PUCHAR	puc_Channel, 
	PUCHAR	puc_Polarity,
	PUSHORT	pus_Value);

/**
 *		@}
 *	@}
 */

#ifdef __cplusplus
	}
#endif
